################################################################################
#
# likelihood.py 		(c) Cameron Liang 
#						University of Chicago
#     				    jwliang@oddjob.uchicago.edu
#
# Defined the likelihood and posterior distribution for the model given the 
# data.      
################################################################################

import numpy as np

from bayesvp.vp_model import continuum_model_flux

np.seterr(all='ignore') # Ignore floating point warnings.

def tophat_prior(model_x, x_left,x_right):
	if model_x >= x_left and model_x < x_right:
		return 0
	else:
		return -np.inf

class Posterior(object):
    """
    Define the natural log of the posterior distribution

    Parameters:
    ----------- 
    config_params: 
        Parameters object defined by the config file
    
    Returns:
    -----------
    lnprob: function
        The posterior distribution as a function of the 
        input parameters given the spectral data
    """
    def __init__(self,config_params):
        self.config_params = config_params

    def lnlike(self,alpha):
        """
        Likelihood assumes flux follow a Gaussian
        
        Returns
        --------
        ln_likelihood: float
            Natural log of the likelihood
        """

        # Flux of the model
        model_flux = continuum_model_flux(alpha,self.config_params)
        
        resid = self.config_params.flux - model_flux

        # Natural log of gaussian likelihood with normalization included 
        ln_likelihood = np.sum(0.5*np.log(2*np.pi*self.config_params.dflux**2)
                              -0.5*resid**2/self.config_params.dflux**2)

        if np.isnan(ln_likelihood):
            return -np.inf
        else:
            return ln_likelihood

    def lnprior(self,alpha):
        """
        Natural Log of the priors for three types of 
        parameters [logN, b, z]

        Note that for redshift z, the ranges are defined 
        to be [mean_z-v/c,mean_z+v/c]  
        """
        
        # Define these for clarity
        min_logN,max_logN = self.config_params.priors[0] 
        min_b,max_b       = self.config_params.priors[1]
        min_z,max_z       = self.config_params.priors[2]

        # Select the parameters that are free or tie (i.e not fixed)
        final_vp_params_type = self.config_params.vp_params_type[~np.isnan(self.config_params.vp_params_flags)]

        sum_logN_prior = 0; sum_b_prior = 0; sum_z_prior = 0

        model_redshifts = []
        
        for i in xrange(len(final_vp_params_type)):
            if final_vp_params_type[i] == 'logN':
                sum_logN_prior += tophat_prior(alpha[i],min_logN,max_logN)
            elif final_vp_params_type[i] == 'b':
                sum_b_prior += tophat_prior(alpha[i],min_b, max_b)
            elif final_vp_params_type[i] == 'z':
                model_redshifts.append(alpha[i])
                sum_z_prior += tophat_prior(alpha[i],min_z,max_z)
        
        # Make sure multiple components do not swap
        model_redshifts = np.array(model_redshifts)
        if not all(sorted(model_redshifts) == model_redshifts):
            sum_z_prior = -np.inf 

        if self.config_params.cont_normalize:
            # linear continuum slope and intercept priors
            contiuum_prior = 0

            for i in range(1,self.config_params.cont_nparams+1):
                contiuum_prior += tophat_prior(alpha[-i],-1,1)

            return sum_logN_prior + sum_b_prior + sum_z_prior + contiuum_prior

        return sum_logN_prior + sum_b_prior + sum_z_prior

    def __call__(self,alpha):
        """
        Posterior distribution

        Returns
        ---------
        lnprob: float
            Natural log of posterior probability
        """

        lp = self.lnprior(alpha)
        
        if np.isinf(lp):
            return -np.inf
        else:
            return np.atleast_1d(lp + self.lnlike(alpha))[0]



################################################################################
#
# config.py 		(c) Cameron Liang 
#						University of Chicago
#     				    jwliang@oddjob.uchicago.edu
#
# Read in and define all of the parameters for voigt profile based on a  
# config file; see write_config.py for producing config file.     
################################################################################

import numpy as np
import os
import re 
import sys
import ntpath

from bayesvp.utilities import get_transitions_params, MyParser


class DefineParams:
    """
    Read and define fitting parameters from 
    config file

    Attributes:
    -----------
    lines: array_like
        All lines containing in config file
    spec_path: str
        Full path to the spectrum file 
    chain_short_fname: str
        Name of the output mcmc chain (with .npy extension)
    self_bvp_test: str
        Determines if it is a test run
    chain_fname: str
    output_path: str
    mcmc_outputpath: str
    data_product_path: str
    data_product_path_files: str
    data_product_path_plots: str
    nwalkers: int
    nsteps: int
    nthreads: int
    wave: array
        Selected region of the input spectral data
    flux: array
    error: array
    wave_begins: array_like
        Selected wavelength regions bounds
    wave_ends: array_like
        Selected wavelength regions bounds
    vp_params: array_like
    transitions_params_array: array_like
    vp_params_type: array_like
        Voigt profile model parameter types, i.e [logN, b, z]
    vp_params_flags: array_like
        indexed flags that indicate the parameter type
    n_component: int
        Number of Components defined by the model
    lsf: array_like
        Line spread function to be convolved with the model
    priors: array_like; shape = (3,2)
        Priors for three types of parameters (logN, b, z)
    """

    def __init__(self,config_fname):

        self.config_fname = config_fname
        self.config_basename = ntpath.basename(config_fname)
        # Read and filter empty lines
        all_lines = filter(None,(line.rstrip() for line in open(config_fname)))

        # Remove commented lines
        self.lines = []
        for line in all_lines:
            if not line.startswith('#') and not line.startswith('!'): 
                self.lines.append(line)    


        ########################################################################
        # Retrieve MCMC parameters from config file
        # --------
        # self.spec_path, self.chain_fname, self.nwalkers, self.nsteps,
        # self.nthreads
        ########################################################################
        
        # continuum model preset to false
        self.cont_normalize  = False; self.cont_nparams = 0

        self.self_bvp_test = False

        # Paths and fname strings
        for line in self.lines:
            line = filter(None,line.split(' '))

            if 'spec_path' in line or 'input' in line or 'spectrum' in line:
                if line[1] == 'test_path_to_spec':
                    self.spec_path = (os.path.dirname(os.path.abspath(__file__)) + 
                                     '/data/example')
                    self.self_bvp_test = True
                else:
                    self.spec_path = line[1]

            elif 'output' in line or 'chain' in line:
                self.chain_short_fname = line[1]

            elif 'continuum' in line or 'cont' in line:
                self.cont_normalize  = True
                self.cont_nparams = int(line[1]) + 1 # n_param = poly degree + 1 (offset)

            elif 'mcmc_params' in line or 'mcmc' in line:
                self.nwalkers = int(line[1])
                self.nsteps   = int(line[2])
                self.nthreads = int(line[3])

                # Default
                self.model_selection = 'bic'      
                self.mcmc_sampler    = 'kombine'

                # Change keys if defined in config 
                for key in line[3:]:
                    if key in ['kombine','emcee']:
                        self.mcmc_sampler = key
                    elif key in ['aic','bic','bf']:
                        self.model_selection = key  
        
        ########################################################################
        # Get the spectral data specified by the config file
        # The spectrum file assumes three column of data with 
        # [wave,flux,error]
        # ---------
        # self.spec_short_fname, self.spec_fname 
        # self.wave, self.flux, self.error 
        ########################################################################
        for line in self.lines: 
            if re.search('%%',line):
                spec_fname_line = line

        spec_data_array = spec_fname_line.split(' ')
        self.spec_short_fname = spec_data_array[1]
        self.spec_fname = self.spec_path + '/' + spec_data_array[1]

        # Select spectral range to fit
        if len(spec_data_array[2:]) % 2 != 0:
            sys.exit('There is an odd number of wavelengths entered in config file.\n Exiting program...')
        else:
            self.wave_begins = np.array(spec_data_array[2:][0::2]).astype(float)
            self.wave_ends   = np.array(spec_data_array[2:][1::2]).astype(float)

            for i in xrange(len(self.wave_begins)):
                if self.wave_begins[i] >= self.wave_ends[i]:
                    sys.exit('Starting wavelength cannot be greater or equal to ending wavelength: (%.3f, %.3f); exiting program...' 
                            % (self.wave_begins[i] ,self.wave_ends[i]))

        wave,flux,dflux = np.loadtxt(self.spec_fname,
                                    unpack=True,usecols=[0,1,2])
        
        # Select regions of interests 
        all_inds = []
        for i in range(len(self.wave_begins)):
            inds = np.where((wave>=self.wave_begins[i]) & (wave<self.wave_ends[i]))[0]
            all_inds.append(inds)

        all_inds   = np.hstack(np.array(all_inds))
        wave = wave[all_inds]; flux = flux[all_inds]; dflux = dflux[all_inds]
        
        # Remove NaN pixels in flux
        inds = np.where((~np.isnan(flux)))
        self.wave = wave[inds]; self.flux = flux[inds]; self.dflux = dflux[inds]

        # Set negative pixels in flux and error 
        inds = np.where((self.flux < 0)); self.flux[inds] = 0
        inds = np.where((self.dflux < 0)); self.dflux[inds] = 0

        if len(self.wave) == 0 or len(self.flux) == 0 or len(self.dflux) == 0:
            raise SystemExit('No data within specified wavelength range.' \
                             'Please check config file and spectrum.')
            

        ########################################################################
        # Get Voigt profile parameters of arbitary number of components
        # specified in the config file.

        # Uses './data/atom.dat' to read in atomic/transition data with format:
        # [atom state rest_wavelength oscillator_strength damping_coeff mass_amu]
        # Users can add additional row to the file for new atomic data

        # --------
        # self.vp_params, self.transitions_params_array, self.vp_params_flags, 
        # self.vp_params_type, self.n_component 
        ########################################################################
        

        # Lines in config file that contain the component parameters
        # i.e atom, state, logN, b, z
        component_lines = []
        for line in self.lines: 
            if re.search('%',line):
                component_lines.append(line)
        component_lines = component_lines[1:] 


        logNs = []; bs = []; redshifts = []
        transitions_params_array = []
        for i in xrange(len(component_lines)):
            line = component_lines[i]
            line = filter(None,line.split(' '))

            atom  = line[1]; state = line[2] # To obtain transition data
            logNs.append(line[3])
            bs.append(line[4])
            redshifts.append(line[5])

            if line[5][-1].isalpha():
                self.redshift = line[5][:-1]
            else:
                self.redshift = line[5]
                

            transitions_params_array.append([])
            # Each component gets a set of all of the transitions data
            for j in xrange(len(self.wave_begins)):
                # each wavelength regions gets all of the transitions
                temp_params = get_transitions_params(atom,state,self.wave_begins[j],self.wave_ends[j],float(self.redshift))
                transitions_params_array[i].append(temp_params)
        
        # Shape = (n_component,n_regions,n_transitions,4) 
        self.transitions_params_array = np.array(transitions_params_array)
        self.vp_params = np.transpose(np.array([logNs,bs,redshifts]))
        self.n_component = len(component_lines) 

        # Define what kind of parameters to get walker initiazation ranges.
        # and for fixing and freeing paramters. 
        
        # Note that this assumed the pattern of parameters. 
        # will update for continuum parameters. 
        vp_params_type = [None]*len(self.vp_params.flatten())
        vp_params_type[::3]  = ['logN'] * (len(vp_params_type[::3]))
        vp_params_type[1::3] = ['b']    * (len(vp_params_type[1::3]))
        vp_params_type[2::3] = ['z']    * (len(vp_params_type[2::3]))
        

        flat_params = self.vp_params.flatten()
        flags = np.zeros(len(flat_params))

        letters = [None]*len(flat_params)
        for i in xrange(len(flat_params)):
            for j in xrange(len(flat_params[i])):
                if flat_params[i][j].isalpha():
                    letters[i] = flat_params[i][j]
        unique_letters = filter(None,list(set(letters)))

        n_free_params_counter = 0
        for i in range(len(letters)):
            if letters[i] == None:
                flags[i] = n_free_params_counter
                n_free_params_counter += 1

        for unique_letter in unique_letters:
            inds = [i for i, x in enumerate(letters) if x == unique_letter]
            if unique_letter.islower(): 
                flags[inds] = n_free_params_counter
                n_free_params_counter += 1
            else:
                for index in inds:
                    flags[index] = None

        # Model uses these to construct sets of (logN, b, z) for each component
        self.vp_params_type  = np.array(vp_params_type)
        self.vp_params_flags = np.array(flags)
        self.n_params        = n_free_params_counter
        if self.cont_normalize:
            self.n_params = self.n_params + self.cont_nparams

        # Make directories for data products
        if self.self_bvp_test:
            # write to local direcotry if it is test to avoid permission issues in bayesvp library location
            self.output_path = '.' + '/bvp_output_z' + str(self.redshift)

        else:
            self.output_path = self.spec_path + '/bvp_output_z' + str(self.redshift)

        self.mcmc_outputpath = self.output_path + '/chains'
        
        self.data_product_path = self.output_path +'/data_products'
        self.data_product_path_files = self.data_product_path + '/ascii'
        self.data_product_path_plots = self.data_product_path + '/plots'
        self.chain_fname = self.mcmc_outputpath + '/' + self.chain_short_fname

        try: 
            os.makedirs(self.mcmc_outputpath)
        except OSError:
            if not os.path.isdir(self.mcmc_outputpath):
                raise

        try: 
            os.makedirs(self.data_product_path_files)
        except OSError:
            if not os.path.isdir(self.data_product_path_files):
                raise

        try: 
            os.makedirs(self.data_product_path_plots)
        except OSError:
            if not os.path.isdir(self.data_product_path_plots):
                raise

        ########################################################################
        # Determine the LSF by specifying LSF filename with 
        # 'database' directory under self.spec_path.    
        # Assumes LSF file contains only 1 column of data
        # -----------
        # lsf 
        ########################################################################

        # Check if LSF is specified in config file
        defined_lsf = False
        for line in self.lines:
            if re.search('lsf',line) or re.search('LSF',line):
                lsf_line = line.split(' ')[1:]
                defined_lsf = True
                if not os.path.isdir(self.spec_path + '/database'):
                    os.mkdir(self.spec_path + '/database')
                    sys.exit('Require LSF file to be in %s' % self.spec_path + '/database\n Exiting program...')
                break

        # Get the LSF function from directory 'database'
        if defined_lsf:
            if len(lsf_line) == len(self.wave_begins):
                self.lsf = []
                for lsf_fname in lsf_line:
                    # assume lsf file has one column 
                    fname = self.spec_path + '/database/' + lsf_fname
                    self.lsf.append(np.loadtxt(fname))
            elif len(lsf_line) == 1:
                for lsf_fname in lsf_line:
                    # assume lsf file has one column 
                    fname = self.spec_path + '/database/' + lsf_fname
                    self.lsf = np.loadtxt(fname)
            else:
                sys.exit('Please check if number of LSF matches wavelength regions. Exiting program...')
        else:
            # Convolve with LSF = 1
            self.lsf = np.ones(len(self.wave_begins))

        
        #######################################################################
        # Read priors and use them for walker initialization 
        # 
        
        # format in config file:
        # logN min_logN max_logN
        # b    min_b    max_b
        # z    center_z min_dv max_dv (range defined by range of velocity) [km/s]  
        # -----------
        # self.priors
        #######################################################################
        
        self.priors = np.zeros((3,2))
        for line in self.lines:
            line = np.array(line.split(' '))
            line = filter(None,line)
            if 'logN' in line:
                if len(line) != 3:
                    sys.exit('Error! In config file, format for logN prior:\n logN min_logN max_logN\nExiting program...')
                self.priors[0] = [float(line[1]),float(line[2])]

            if 'b' in line:
                if len(line) != 3:
                    sys.exit('Error! In config file, format for b prior:\n z min_b max_b\nExiting program...')
                self.priors[1] = [float(line[1]),float(line[2])]

            if 'z' in line:
                c = 299792.458 # speed of light [km/s]
                if len(line) == 4:
                    center_z,min_dv,max_dv = float(line[1]),float(line[2]),float(line[3])
                    if min_dv == max_dv:
                        min_dv = -min_dv
                    min_z,max_z = center_z+min_dv/c,center_z+max_dv/c
                elif len(line) == 3:
                    center_z,dv = float(line[1]),float(line[2])
                    min_z,max_z = center_z-dv/c,center_z+dv/c
                else:
                    sys.exit('Error! In config file, format for z prior:\n z center_z |min_dv| |max_dv|\nor\n' + 
                            ' z center_z dv\nThe latter option will use +/- dv [km/s]')
                self.priors[2] = [min_z,max_z]
    
    def print_config_params(self):

        # First copy the original config file to output path
        with open(self.output_path + '/' + self.config_basename,'w') as f_config:
            f_config.writelines('\n'.join(self.lines))

        f_logging = open(self.output_path + '/config.log','w')

        f_logging.write('Config file: %s\n'     % self.config_fname)
        f_logging.write('Spectrum Path: %s\n'     % self.spec_path)
        f_logging.write('Spectrum name: %s\n'     % self.spec_short_fname)
        
        f_logging.write('Fitting %i components with transitions:\n' % self.n_component)
        for i in xrange(len(self.transitions_params_array)):
            for j in xrange(len(self.transitions_params_array[i])):
                if not np.isnan(self.transitions_params_array[i][j]).any():
                    for k in xrange(len(self.transitions_params_array[i][j])):
                        rest_wavelength = self.transitions_params_array[i][j][k][1]
                        f_logging.write('    Transitions Wavelength: %.3f\n' % rest_wavelength)
                else:
                    sys.exit('No transitions satisfy the wavelength regime for fitting;Check input wavelength boundaries')

        f_logging.write('Selected data wavelegnth region:\n')
        for i in xrange(len(self.wave_begins)):
            f_logging.write('    [%.3f, %.3f]\n' % (self.wave_begins[i],self.wave_ends[i])) 
        
        if self.cont_normalize:
            f_logging.write('Continuum polynomial degree: %i\n'     % (self.cont_nparams - 1))
        f_logging.write('MCMC Sampler: %s\n' % self.mcmc_sampler)
        f_logging.write('Model selection method: %s\n' % self.model_selection)
        f_logging.write('Walkers,steps,threads : %i,%i,%i\n' % (self.nwalkers,self.nsteps,self.nthreads))
        f_logging.write('Priors: ')
        f_logging.write('logN: [min,   max] = [%.3f, %.3f]\n' % (self.priors[0][0],self.priors[0][1]))
        f_logging.write('b:            [min,   max] = [%.3f, %.3f]\n' % (self.priors[1][0],self.priors[1][1]))
        f_logging.write('redshift:     [min,   max] = [%.5f, %.5f]\n' % (self.priors[2][0],self.priors[2][1]))
        f_logging.close()
    
def main():
    parser = MyParser()
    parser.add_argument('config_fname',help="full path to config filename", nargs='?')
    parser.add_argument("-t", "--test",help="test for reading config file",
                        action="store_true")
    parser.add_argument("-v", "--verbose",help="print config summary to file ",
                        action="store_true")
    

    if len(sys.argv)==1:
        parser.print_help(sys.stderr)
        sys.exit(1)
    
    args = parser.parse_args()

    if args.test:
        from bayesvp.utilities import get_bayesvp_Dir
        path = get_bayesvp_Dir()
        config_fname = path + '/data/example/config_OVI.dat'
        config_params = DefineParams(config_fname)
        config_params.print_config_params()

    if args.config_fname is None: 
        pass
    else:
        if os.path.isfile(args.config_fname):
            config_params = DefineParams(args.config_fname)
            if args.verbose:
                config_params.print_config_params()
        else:
            sys.exit('Config file does not exist:\n %s' % args.config_fname)

if __name__ == '__main__':
    sys.exit(main() or 0)



###############################################################################
#
# bvp_plot_model.py     (c) Cameron Liang 
#						University of Chicago
#     				    jwliang@oddjob.uchicago.edu
#
# Process model outputs such as corner plot, GR plot, and associated    
# ascii data files. 
# 
###############################################################################


import numpy as np
import sys
import os
import matplotlib.pyplot as plt
from scipy.interpolate import interp1d


from bayesvp.vp_model import continuum_model_flux
from bayesvp.utilities import write_mcmc_stats, compute_burnin_GR
from bayesvp.utilities import MyParser,extrapolate_pdf
from bayesvp.config import DefineParams


class ProcessModel:

    def __init__(self,config_param):

        self.config_param = config_param
        mcmc_chain_fname = config_param.chain_fname + '.npy'
        mcmc_chain = np.load(mcmc_chain_fname)

        # MCMC chains
        self.burnin = compute_burnin_GR(config_param.chain_fname + '_GR.dat')
        self.burned_in_samples = mcmc_chain[self.burnin:, :, :].reshape((-1, self.config_param.n_params))


        # Best fit model parameters and spectrum flux
        self.best_fit_params = np.median(self.burned_in_samples,axis=0)
        self.model_flux = continuum_model_flux(self.best_fit_params,config_param)

        # Get parameters labels/legends for figures
        logN_counter = 0 
        b_counter    = 0
        z_counter    = 0
        cont_param_counter = 0

        self.plot_param_labels = []
        self.gr_param_label = []
        self.ascii_filename_label = []
        for n in range(self.config_param.n_params):
            if n < self.config_param.n_params-self.config_param.cont_nparams:
                if self.config_param.vp_params_type[n] == 'logN':
                    logN_counter += 1
                    temp_label = r'$\log N_{%s}/\rm{cm^{-2}}$' % str(logN_counter)
                    self.plot_param_labels.append(temp_label)
                    self.gr_param_label.append(r'$logN_{%s}$' % str(logN_counter))
                    self.ascii_filename_label.append('logN_%s' % str(logN_counter))
                elif self.config_param.vp_params_type[n] == 'b':
                    b_counter += 1
                    temp_label = r'$b_{%s}/\rm{km s^{-1}}$' % str(b_counter)
                    self.plot_param_labels.append(temp_label)
                    self.gr_param_label.append(r'$b_{%s}$' % str(b_counter))
                    self.ascii_filename_label.append('b_%s' % str(b_counter))
                elif self.config_param.vp_params_type[n] == 'z':
                    z_counter += 1 
                    temp_label = r'$z_{%s}/10^{-5}$' % str(z_counter)
                    self.plot_param_labels.append(temp_label)
                    self.gr_param_label.append(r'$z_{%s}$' % str(z_counter))
                    self.ascii_filename_label.append('z_%s' % str(z_counter))
            else:
                cont_param_counter += 1
                temp_label = r'$a_{%s}$' % str(cont_param_counter)
                self.plot_param_labels.append(temp_label)
                self.ascii_filename_label.append(temp_label)
                self.gr_param_label.append(temp_label)

        self.plot_param_labels   = np.array(self.plot_param_labels)
        self.gr_param_label = np.array(self.gr_param_label)
        self.ascii_filename_label = np.array(self.ascii_filename_label)


    def save_marginalized_pdf(self,n):
        """Plot and save the marginalized distribution of voigt profile parameters"""
        
        param_type  = self.config_param.vp_params_type[n]
        param_pos   = self.config_param.vp_params_flags[n]
        param_label = self.ascii_filename_label[n] 
        x = self.params_pdfs[n][0]; log_pdf = self.params_pdfs[n][1]

        # write to disk
        output_name_prefix = self.config_param.data_product_path_files+'/pdf_'+ \
                            param_label +'_'+ self.config_param.chain_short_fname 
        
        f = open(output_name_prefix + '.dat','w')
        if 'z' in param_label:
            f.write('# %sx1e5\t%s\n' % (param_type, 'log10(pdf)'))
        else:
            f.write('# %s\t%s\n' % (param_type, 'log10(pdf)'))
        for i in xrange(len(x)):
            if not np.isinf(log_pdf[i]):
                f.write('%.4f\t%.16f\n' % (x[i], log_pdf[i]))
        f.close()

        print('Written %s' % output_name_prefix + '.dat')


    def spline_binned_pdf(self,bins = 30, interp_kind = 'linear'):
        """
        Convert from MCMC samples to PDF by interpolating 
        the histogram; extrapolate logN if needed. 
        """

        self.params_pdfs = []
        for n in range(self.config_param.n_params):
            bin_step_size = 0.01
            if 'z' in self.ascii_filename_label[n]:
                pdf, edges = np.histogram(self.burned_in_samples[:,n]*1e5,density=1,bins=bins)
            else:
                pdf, edges = np.histogram(self.burned_in_samples[:,n],density=1,bins=bins)

            f = interp1d(edges[1:],pdf,kind=interp_kind)
            x = np.arange(min(edges[1:]),max(edges[1:]),bin_step_size)

            if 'logN' in self.ascii_filename_label[n]:
                new_bounds = [0.0,23.0]
                x_tmp,log_pdf_tmp = extrapolate_pdf(x,f(x),new_bounds[0],new_bounds[1],
                                    bin_step_size)
                self.params_pdfs.append([x_tmp,log_pdf_tmp])

            elif 'b' in self.ascii_filename_label[n]:
                new_bounds = [0.0,500.0]
                x_tmp,log_pdf_tmp = extrapolate_pdf(x,f(x),new_bounds[0],new_bounds[1],
                                    bin_step_size)
                self.params_pdfs.append([x_tmp,log_pdf_tmp])
            else:
                min_x = np.min(x) - 100
                max_x = np.max(x) + 100
                x_tmp,log_pdf_tmp = extrapolate_pdf(x,f(x),min_x,max_x,
                                    bin_step_size)
                self.params_pdfs.append([x_tmp,log_pdf_tmp])

            if n < self.config_param.n_params-self.config_param.cont_nparams:
                self.save_marginalized_pdf(n)

        self.params_pdfs = np.array(self.params_pdfs)

    def plot_model_comparison(self,redshift,dv,central_wave=None):
        """ Plot best fit model onto spectrum for visual inspection 
        """
        c = 299792.485 # [km/s]

        if central_wave == None:
            # Use the first transition as the central wavelength
            central_wave = self.config_param.transitions_params_array[0][0][0][1]
        else:
            central_wave = float(central_wave)

        obs_spec_wave = self.config_param.wave / (1+redshift) 
        obs_spec_dv = c*(obs_spec_wave - central_wave) / central_wave
        plt.rc('text', usetex=True)

        plt.figure(1)
        plt.step(obs_spec_dv,self.config_param.flux,'k',label=r'$\rm Data$')
        plt.step(obs_spec_dv,self.model_flux,'b',lw=2,label=r'$\rm Best\,Fit$')
        plt.step(obs_spec_dv,self.config_param.dflux,'r')
        plt.axhline(1,ls='--',c='g',lw=1.2)
        plt.axhline(0,ls='--',c='g',lw=1.2)
        plt.axvline(0,ls='--',c='g',lw=1.2)
        plt.ylim([-0.1,1.4])
        plt.xlim([-dv,dv])
        plt.xlabel(r'$dv\,[\rm km/s]$',fontsize=15)
        plt.ylabel(r'$\rm Normalized\,Flux$',fontsize=15)
        plt.legend(loc='best')
        
        output_name = (self.config_param.data_product_path_plots + '/modelspec_'
                      + self.config_param.chain_short_fname + '.png')
        plt.savefig(output_name,bbox_inches='tight',dpi=100)
        plt.clf()
        print('Written %s' % output_name)

    def corner_plot(self,truths=None):
        """
        Make triangle plot for visuaizaliton of the 
        multi-dimensional posterior
        """
        import corner

        self.truths = truths
        if self.truths:
            self.truths = np.array(truths)
            if len(truths) != self.config_param.n_params:
                sys.exit('Number of true values (%i) should equal number of parameters (%i)\n Exiting program...' 
                        % (len(truths),self.config_param.n_params))

        for n in range(self.config_param.n_params):
            if (n < (self.config_param.n_params-self.config_param.cont_nparams) 
                and self.config_param.vp_params_type[n] == 'z'):
                self.burned_in_samples[:,n] = self.burned_in_samples[:,n] * 1e5
                
                if self.truths:
                    self.truths[n] = self.truths[n] * 1e5


        plt.figure(1)
        fig = corner.corner(self.burned_in_samples,bins=30,quantiles=(0.16,0.5, 0.84),
            truths=self.truths,truth_color='g',use_math_text=True,labels=self.plot_param_labels,
            show_titles=True,title_kwargs={"fontsize": 16})

        output_name = self.config_param.data_product_path_plots + '/corner_' + \
                      self.config_param.chain_short_fname + '.png'
        
        plt.savefig(output_name,bbox_inches='tight')
        plt.clf()

        print('Written %s' % output_name)

    def plot_gr_indicator(self):
        """
        Make plot for the evolution of GR indicator 
        as a function of steps
        """
        gr_fname = self.config_param.chain_fname + '_GR.dat'
        data = np.loadtxt(gr_fname,unpack=True)
        steps = data[0]; grs = data[1:]
        
        plt.figure(1,figsize=(6,6))
        for i in xrange(len(grs)):
            plt.plot(steps,grs[i],lw=1.5,label=self.gr_param_label[i])
            
        plt.legend(loc='best')
        plt.xscale('log')

        plt.xlabel(r'$N(\rm{steps})$',fontsize=15)
        plt.ylabel(r'$R_{\rm GR}$',fontsize=15)

        output_name = self.config_param.data_product_path_plots + '/GR_' + \
                      self.config_param.chain_short_fname + '.png'
        plt.savefig(output_name,bbox_inches='tight',dpi=100)
        plt.clf()

        print('Written %s' % output_name)


    def write_model_spectrum(self):
        """
        Write to file the input segment of spectrum for fitting and the 
        best fit model flux 
        """
        output_fname = self.config_param.data_product_path_files +  \
                    '/spec_' + self.config_param.chain_short_fname+'.dat' 
        
        np.savetxt(output_fname,np.c_[self.config_param.wave,
                                      self.config_param.flux, 
                                      self.config_param.dflux,
                                      self.model_flux],header='wave\tflux\terror\tmodel\n')
        print('Written %s' % output_fname)
        
    def write_model_summary(self):
        """
        Write to file the confidence levels of all the parameters
        in the best fit model
        """
        mcmc_chain_fname = self.config_param.chain_fname + '.npy'
        output_summary_fname = self.config_param.data_product_path_files + \
                        '/params_'+self.config_param.chain_short_fname+'.dat'
        write_mcmc_stats(self.config_param,output_summary_fname)


def main():

    parser = MyParser()
    parser.add_argument('config_fname',help="full path to config filename", nargs='?',type=str)
    parser.add_argument('redshift',help="central redshift for plotting spectrum", nargs='?',type=float)
    parser.add_argument('dv',help="velocity range for spectrum (+/-dv)", nargs='?',type=float)
    parser.add_argument("--truths", nargs='+', type=float)
    parser.add_argument('-t', "--test",help="plot data product of test", action="store_true")    
    
    args = parser.parse_args()

    if len(sys.argv)<4 and not args.test:
        parser.print_help(sys.stderr)
        sys.exit(1)

    if args.test:
        from bayesvp.utilities import get_bayesvp_Dir
        path = get_bayesvp_Dir()
        config_fname = path + '/data/example/config_OVI.dat'

        config_param = DefineParams(config_fname)
        args.redshift = 0.0; args.dv = 300.0
    
    else:
        if os.path.isfile(args.config_fname):
            config_param = DefineParams(args.config_fname)
        else:
            sys.exit('Config file does not exist:\n %s' % args.config_fname)

    output_model = ProcessModel(config_param)
    output_model.spline_binned_pdf()
    output_model.plot_model_comparison(args.redshift,args.dv)
    output_model.write_model_summary()
    output_model.write_model_spectrum()
    output_model.plot_gr_indicator()
    output_model.corner_plot(args.truths)


if __name__ == '__main__':
    sys.exit(main() or 0)




################################################################################
#
# mcmc_setup.py 		(c) Cameron Liang 
#						University of Chicago
#     				    jwliang@oddjob.uchicago.edu
#
# Perform maximum likelihood fit to data given the config file specification 
# and output the MCMC chains     
################################################################################


import numpy as np
import sys
import os
import logging


from bayesvp.config import DefineParams
from bayesvp.utilities import determine_autovp,model_info_criterion,\
					  compare_model,gr_indicator 


def _create_walkers_init(config_params):
	"""
	Initialize walkers with the free parameters. 
	Note that the initialization sets the number of parameters
	in the sampler. This initialization depends on the config file. 

	Parameters
	-----------
	config_params: obj
		Parameter object defined by the config file; Following 
		attributes are used in creating walkers initialization. 
	
		vp_params_flags: array_like
			Determines whether or not a parameter is fixed, 
			free or tied. If unique int -> free; if repeated 
			int -> tied; if None -> fixed.
		vp_params_type: array_like
			Determine the types of parameters (logN, b, z) for 
			initializationof the ranges.  
		nwalkers: int
			Number of walkers to run in MCMC.
	
	Returns
	-----------
	p0.T: array; shape = (nwalkers,ndim)
		The starting point of the walkers at n dimensions, where
		n = number of parameters. These will then walk/sample around 
		the parameters space based on the MCMC algorithm used. 
	"""

	temp_flags = config_params.vp_params_flags[~np.isnan(config_params.vp_params_flags)]
	n_params =  len(list(set(temp_flags)))

	# Get all the free parameters types
	final_vp_params_type = config_params.vp_params_type[~np.isnan(config_params.vp_params_flags)]

	p0 = np.zeros((n_params,config_params.nwalkers))
	for i in xrange(n_params):
		# require if-conditions for specific parameters 
		if final_vp_params_type[i] == 'logN':
			p0[i] = np.random.uniform(config_params.priors[0][0],
									config_params.priors[0][1],
									size=config_params.nwalkers)
		elif final_vp_params_type[i] == 'b':
			p0[i] = np.random.uniform(config_params.priors[1][0],
									config_params.priors[1][1],
									size=config_params.nwalkers)
		elif final_vp_params_type[i] == 'z':
			p0[i] = np.random.uniform(config_params.priors[2][0],
									config_params.priors[2][1],
									size=config_params.nwalkers)

	if config_params.cont_normalize:
		p1 = np.zeros((config_params.cont_nparams,config_params.nwalkers))

		for i in range(config_params.cont_nparams):
			p1[i] = np.random.uniform(-1,1,size=config_params.nwalkers )

		p = np.concatenate((p0,p1),axis=0)
		return np.transpose(p)

	return np.transpose(p0)



def bvp_mcmc_single(config_params,chain_filename_ncomp = None):
	"""
	Run MCMC and save the chain based on parameters defined 
	in config file. 

	Parameters
	-----------
	config_params: obj
		Parameter object defined by the config file; Following 
		attributes are used in creating walkers initialization.
	chain_filename_ncomp: str
		Output filename without extention; '.npy' is assumed 
		added later

	Returns
	-----------
	chains: python format (.npy) binary file 
		One chain for the specified MCMC run. Use np.load
		to load the n-dim array into memory for manipulation.
	"""
	from bayesvp.likelihood import Posterior

	if chain_filename_ncomp is None:
		chain_filename_ncomp = config_params.chain_fname

	# define the MCMC parameters.
	p0 = _create_walkers_init(config_params)
	ndim = np.shape(p0)[1]


	# Define the natural log of the posterior 
	lnprob = Posterior(config_params)

	if config_params.mcmc_sampler.lower() == 'emcee':
		import emcee
		sampler = emcee.EnsembleSampler(config_params.nwalkers, ndim, lnprob, 
										threads=config_params.nthreads)
		sampler.run_mcmc(p0,config_params.nsteps)
		np.save(chain_filename_ncomp + '.npy', np.swapaxes(sampler.chain,0,1))
	
	elif config_params.mcmc_sampler.lower() == 'kombine':
		import kombine
		sampler = kombine.Sampler(config_params.nwalkers, ndim, lnprob,
								  processes=config_params.nthreads)
		
		# First do a rough burn in based on accetance rate.
		p_post_q = sampler.burnin(p0)
		p_post_q = sampler.run_mcmc(config_params.nsteps)
		np.save(chain_filename_ncomp + '.npy', sampler.chain)
	
	else:
		sys.exit('Error! No MCMC sampler selected.\nExiting program...')


	# Compute Gelman-Rubin Indicator
	dnsteps = int(config_params.nsteps*0.05)
	n_steps = []; Rgrs = []	
	for n in xrange(dnsteps,config_params.nsteps):
		if n % dnsteps == 0:
			Rgrs.append(gr_indicator(sampler.chain[:n,:,:]))
			n_steps.append(n)
	n_steps = np.array(n_steps)
	Rgrs    = np.array(Rgrs)

	np.savetxt(chain_filename_ncomp+ '_GR.dat',np.c_[n_steps,Rgrs],fmt='%.5f',
				header='col1=steps\tcoln=gr_indicator')

	return 


def bvp_mcmc(config_fname,print_config=False):
	"""
	Run fixed number of component fit specified by the 
	config file or make copies of the configs and run up 
	to the maximum number of components until the best model 
	is found.

	Parameters:
	-----------
	config_fname: str
		Full path + the file name
		See ./readme.md for detaieded structure of the config file

	Returns
	-----------
	chains: python format (.npy) binary file 
		One or more chains for each MCMC run.  
	"""

	# Determine if config is set to automatically choose the best model 
	# If so, create multiple configs file specified by n_component_max
	auto_vp, n_component_min, n_component_max = determine_autovp(config_fname)

	if auto_vp:
		model_evidence = np.zeros(n_component_max-n_component_min+1)
		for n in xrange(n_component_max-n_component_min+1):

			# Get new config filename; 
			basename_with_path, config_extension = os.path.splitext(config_fname)
			config_fname_ncomp = (basename_with_path + str(n+n_component_min)
								+ config_extension)

			# Load config parameter object 
			config_params = DefineParams(config_fname_ncomp)

			# Run MCMC
			bvp_mcmc_single(config_params,config_params.chain_fname)

			# compute values of aic, bic or bf
			model_evidence[n] = model_info_criterion(config_params)
			components_count  = np.arange(n_component_min,n_component_max+1)

			# compare with the previous fit 
			if 0 < n <= n_component_max-1:

				if config_params.model_selection.lower() in ('odds','bf'):   
					index = np.where(model_evidence[:n+1] == np.max(model_evidence[:n+1]))[0]

				elif config_params.model_selection.lower() in ('aic','bic'):
					index = np.where(model_evidence[:n+1] == np.min(model_evidence[:n+1]))[0]

				# Compare BIC/AIC/Odds ratios
				if compare_model(model_evidence[n-1], model_evidence[n],config_params.model_selection):
					np.savetxt(config_params.mcmc_outputpath  + '/' + 
								config_params.model_selection + '_' + 
								config_params.chain_short_fname[:-1]+'.dat',
								np.c_[components_count[:n+1],model_evidence[:n+1]],
								fmt=('%d','%.4f'),header='nComponents\tValues')
					break
	else:
		# Load config parameter object 
		config_params = DefineParams(config_fname)

		if print_config:
			config_params.print_config_params()
		
		# Run fit as specified in config
		bvp_mcmc_single(config_params)	



################################################################################
#
# utilities.py 		(c) Cameron Liang 
#						University of Chicago
#     				    jwliang@oddjob.uchicago.edu
#
# Utility functions used throughout BayesVP      
################################################################################

import numpy as np
import re
import os
import sys
import argparse
from scipy.special import gamma


###############################################################################
# Model Comparisons: Bayesian Evidence / BIC / AIC  
###############################################################################

def determine_autovp(config_fname):
	"""
	Determine based on config file if automatic 
	mode is chosen for vpfit. If auto = True, then 
	reproduce the (n_max - n_min) number of files with 
	that number components
	"""
	def replicate_config(config_fname,normal_lines, 
						component_line,n_component):

		basename_with_path, config_extension = os.path.splitext(config_fname)
		new_config_fname = (basename_with_path + str(n_component)
							 + config_extension)

		f = open(new_config_fname,'w')
		for line in normal_lines:
			temp_line = line.split(' ')
			
			# Add the number of component at the end of output chain filename
			if 'output' in temp_line or 'chain' in temp_line:
				output_name = temp_line[1] + str(n_component)
				f.write(temp_line[0] + ' '); f.write(output_name)
				f.write('\n')
			else:
				f.write(line); f.write('\n')
		
		for line in component_line:
			for n in xrange(1,n_component+1):
				f.write(line); f.write('\n')
		f.close()

	# Read and filter empty lines
	all_lines = filter(None,(line.rstrip() for line in open(config_fname)))

	normal_lines = []   # all lines except line with '!'
	component_line = [] # lines with one '%'
	auto_vp = False; n_component_max = 1; n_component_min = 1
	for line in all_lines:
		if line.startswith('!'): 
			if re.search('auto', line) or re.search('AUTO', line): 
				line = line.split(' ')
				if len(line) == 3:
					n_component_min = 1 
					n_component_max = int(line[2])
				elif len(line) == 4: 
					n_component_min = int(line[2])
					n_component_max = int(line[3])
				auto_vp = True

		elif re.search('%',line):
			if line.split(' ')[0] == '%':
				component_line.append(line)
			else:
				normal_lines.append(line)
		else:
			normal_lines.append(line)

	if auto_vp:
		# Produce Config files
		for n in xrange(n_component_min,n_component_max+1):
			replicate_config(config_fname,normal_lines,component_line,n)

	return auto_vp, n_component_min, n_component_max


def model_info_criterion(obs_spec_obj):
	"""
	Use either aic,bic or bayes factor for model selection

	Aikake Information Criterion (AIC); 
	Bayesian Information Criterion (BIC); 
	see Eqn (4.17) and Eqn (5.35) Ivezic+ "Statistics, 
	Data Mining and Machine Learning in Astronomy" (2014)
	"""

	if obs_spec_obj.model_selection.lower() in ('odds','bf'):
		return local_density_bf(obs_spec_obj) 

	else:
		from Likelihood import Posterior
		# Define the posterior function based on data
		lnprob = Posterior(obs_spec_obj)

		data_length = len(obs_spec_obj.flux)

		chain = np.load(obs_spec_obj.chain_fname + '.npy')
		n_params = np.shape(chain)[-1]
		samples = chain.reshape((-1,n_params))
		medians = np.median(samples,axis=0) # shape = (n_params,)

		log10_L = lnprob(medians)
		lnL = log10_L /np.log10(2.7182818)
		if obs_spec_obj.model_selection.lower() == 'aic':
			return -2*lnL + 2*n_params + 2*n_params*(n_params+1)/(np.log(data_length) - n_params - 1)
		elif obs_spec_obj.model_selection.lower() == 'bic':
			return -2*lnL + n_params*np.log(data_length)
	
		else:
			print('model_selection is not defined to either be aic or bic')
			sys.exit()

def estimate_bayes_factor(traces, logp, r=0.05):
	"""
	Esitmate Odds ratios in a random subsample of the chains in MCMC
	AstroML (see Eqn 5.127, pg 237)
	"""
	from sklearn.neighbors import BallTree
	
	ndim, nsteps = traces.shape # [ndim,number of steps in chain]

	# compute volume of a n-dimensional (ndim) sphere of radius r
	Vr = np.pi ** (0.5 * ndim) / gamma(0.5 * ndim + 1) * (r ** ndim)

	# use neighbor count within r as a density estimator
	bt = BallTree(traces.T)
	count = bt.query_radius(traces.T, r=r, count_only=True)

	# BF = N*p/rho
	bf = logp + np.log(nsteps) + np.log(Vr) - np.log(count) #log10(bf)

	p25, p50, p75 = np.percentile(bf, [25, 50, 75])
	return p50, 0.7413 * (p75 - p25)

########################################################################

def local_density_bf(obs_spec_obj):
	"""
	Bayes Factor: L(M) based on local density estimate
	See (5.127) in Ivezic+ 2014 (AstroML)

	Assuming we need only L(M) at a given point.
	"""
	from Likelihood import Posterior
	from kombine.clustered_kde import ClusteredKDE

	# Define the posterior function based on data
	lnprob = Posterior(obs_spec_obj)
	chain = np.load(obs_spec_obj.chain_fname + '.npy')
	n_params = np.shape(chain)[-1]
	samples = chain.reshape((-1,n_params))

	# KDE estimate of the sample
	sample_fraction = 0.2
	n_sample = (obs_spec_obj.nsteps)*sample_fraction
	ksample = ClusteredKDE(samples)
	sub_sample = ksample.draw(n_sample) 

	logp = np.zeros(n_sample)
	for i in xrange(n_sample):
		logp[i] = lnprob(sub_sample[i])

	bf,dbf = estimate_bayes_factor(sub_sample.T,logp)
	return bf

def compare_model(L1,L2,model_selection):
	"""
	Compare two Models L(M1) = L1 and L(M2) = L2. 
	if return True L1 wins; otherwise L2 wins. 

	For Bayes Factor/Odds ratio, it is the log10(L1/L2) being 
	compared. 
	"""

	if model_selection in ('bic', 'aic'):
		return L1 <= L2
	elif model_selection in ('odds', 'BF','bf'):
		return L1-L2 >= 0

###############################################################################
# Others 
###############################################################################
def bic_gaussian_kernel(chain_fname,data_length):
	"""
	Bayesian information criterion
	Only valid if data_length >> n_params

	# Note that bandwidth of kernel is set to 1 universally
	"""
	from sklearn.neighbors import KernelDensity

	chain = np.load(chain_fname + '.npy')
	n_params = np.shape(chain)[-1]
	samples = chain.reshape((-1,n_params))
	kde = KernelDensity(kernel='gaussian',bandwidth=1).fit(samples)

	# Best fit = medians of the distribution
	medians = np.median(samples,axis=0) # shape = (n_params,)
	medians = medians.reshape(1,-1) 	# Reshape to (1,n_params)
	
	log10_L = float(kde.score_samples(medians)) 
	lnL = log10_L /np.log10(2.7182818)
	 
	return -2*lnL + n_params*np.log(data_length)


###############################################################################
# Process chain
###############################################################################

def compute_stats(x):
	xmed = np.median(x); xm = np.mean(x); xsd = np.std(x)
	xcfl11 = np.percentile(x,16); xcfl12 = np.percentile(x,84)
	xcfl21 = np.percentile(x,2.5); xcfl22 = np.percentile(x,97.5)	
	return xmed,xm,xsd,xcfl11, xcfl12, xcfl21,xcfl22
    

def read_mcmc_fits(config_params_obj,para_name):
    
	my_dict = {'logN':0, 'b':1,'z':2}
	col_num = my_dict[para_name]
	chain = np.load(config_params_obj.chain_fname + '.npy')
	burnin = compute_burnin_GR(config_params_obj.chain_fname + '_GR.dat')
	x = chain[burnin:,:,col_num].flatten()
	xmed,xm,xsd,xcfl11, xcfl12, xcfl21,xcfl22 = compute_stats(x)
	return xmed 

def write_mcmc_stats(config_params_obj,output_fname):
	chain = np.load(config_params_obj.chain_fname + '.npy')
	burnin = compute_burnin_GR(config_params_obj.chain_fname + '_GR.dat')
	
	f = open(output_fname,'w')
	f.write('x_med\tx_mean\tx_std\tx_cfl11\tx_cfl12\t x_cfl21\tx_cfl22\n')
	
	n_params = np.shape(chain)[-1]
	for i in xrange(n_params):
		x            = chain[burnin:,:,i].flatten()
		output_stats = compute_stats(x)
		f.write('%f\t%f\t%f\t%f\t%f\t%f\t%f\n' % 
				(output_stats[0],output_stats[1],
				 output_stats[2],output_stats[3],
				 output_stats[4],output_stats[5],
				 output_stats[6]))
		
	f.close()
	print('Written %s' % output_fname)

	return

def extrapolate_pdf(x,pdf,left_boundary_x,right_boundary_x,x_stepsize,slope=10):
    """ 
    Extrapolate the log10(pdf) outside the range of (min_x,max_x) with 
    some logarithmic slope 
    """

    log_pdf = np.log10(pdf)
    min_x = min(x); max_x = max(x)
    #x_stepsize = np.median(x[1:]-x[:-1])

    entered_left_condition = False
    if min_x >= left_boundary_x:

        # equation of a line with +10 slope going down to the left.
        left_added_x = np.arange(left_boundary_x,min_x,x_stepsize) 
        m = slope; b = log_pdf[0] - m*min_x
        left_pdf = m*left_added_x + b 
    
        # Combine the two segments    
        new_x = np.concatenate((left_added_x,x))
        log_pdf = np.concatenate((left_pdf,log_pdf))
        
        entered_left_condition = True
    
    if max_x <= right_boundary_x:
        
        # Equation of a line with -10 slope going down to the right.
        right_added_x = np.arange(max_x,right_boundary_x,x_stepsize)
        m = -slope; b = log_pdf[-1] - m*max_x
        right_pdf = m*right_added_x + b
        
        # In case new_x is not defined yet if not entered previous condition
        if entered_left_condition:
            new_x = np.concatenate((new_x,right_added_x))
        else:
            new_x = np.concatenate((x,right_added_x))
        log_pdf = np.concatenate((log_pdf,right_pdf))        

    # Normalize the pdf
	pdf_tmp  = 10**log_pdf/np.sum((10**log_pdf)*(x_stepsize))
	inds = np.where(pdf_tmp<=0)[0]; pdf_tmp[inds] = np.min(pdf_tmp)
    log_pdf = np.log10(pdf_tmp)
    return new_x, log_pdf



###############################################################################
# Line Spread Function 
###############################################################################

def gaussian_kernel(std):
    var = std**2
    size = 8*std +1 # this gaurantees size to be odd.
    x = np.linspace(-100,100,size)
    norm = 1/(2*np.pi*var)
    return norm*np.exp(-(x**2/(2*std**2)))

def convolve_lsf(flux,lsf):
	if len(flux) < len(np.atleast_1d(lsf)):
		# Add padding to make sure to return the same length in flux.
		padding = np.ones(len(lsf)-len(flux)+1)
		flux = np.hstack([padding,flux])
    	
		conv_flux = 1-np.convolve(1-flux,lsf,mode='same') /np.sum(lsf)
		return conv_flux[len(padding):]

	else:
		# convolve 1-flux to remove edge effects wihtout using padding
		return 1-np.convolve(1-flux,lsf,mode='same') /np.sum(lsf)

###############################################################################
# Convergence 
###############################################################################

def gr_indicator(chain):
	"""
	Gelman-Rubin Indicator

	Parameters:
	-----------
	chain: array_like
		Multi-dimensional array of the chain with shape (nsteps,nwalkers,ndim)

	Returns
	-----------
	Rgrs: array_like
		Gelman-Rubin indicator with length of (ndim)
	"""
	nsteps,nwalkers,ndim = np.shape(chain)
	nsteps = float(nsteps); nwalkers = float(nwalkers)
    
	Rgrs = np.zeros(ndim)
	for n in xrange(ndim):
		x = chain[:,:,n]
		# average of within-chain variance over all walkers
		W = np.mean(np.var(x,axis=0)) # i.e within-chain variance
		mean_x_per_chain = np.mean(x,axis=0)
		mean_x = np.mean(mean_x_per_chain) 

		# Variance between chains 
		B = nsteps*np.sum((mean_x_per_chain - mean_x)**2) / (nwalkers-1)
		var_per_W = 1 - 1./nsteps + B/(W*nsteps) 

		Rgrs[n] = ((nwalkers+1)/nwalkers) * var_per_W - (nsteps-1)/(nwalkers*nsteps)
        
	return Rgrs 

def compute_burnin_GR(gr_fname,gr_threshold=1.005):
	"""
	Calculate the steps where the chains are 
	converged given a Gelman-Rubin (GR) threshod. 

	Parameters
	----------
	gr_fname:str
		Full path to the GR file with first column as steps
		and the rest as the values of GR for each model parameter
	gr_threshold:float
		The threshold for chains to be considered as converged.
		Default value = 1.01
	Returns
	----------
	burnin_steps: int
		The step number where the least converged parameter has 
		converged; (maxiumn of the steps of all parameters) 
	"""
	data = np.loadtxt(gr_fname,unpack=True)
	steps = data[0]; grs = data[1:]
	indices = np.argmax(grs<=gr_threshold,axis=1)
	# burnin_steps
	return int(np.max(steps[indices]))


###############################################################################
# Others
###############################################################################

def straight_line(x,m,b):
	return (m*x + b) #/ np.mean((m*x + b))

def linear_continuum(wave,flux,m,b):
	continuum = straight_line(wave,m,b)
	return flux * continuum

def get_transitions_params(atom,state,wave_start,wave_end,redshift):
	"""
	Extract the ionic and tranisiton properties based on the atom and 
	the state, given the redshift and observed wavelength regime.

	Parameters:
	----------
	atom: str
		names of atoms of interests (e.g., 'H', 'C','Si')
	state: str
		ionization state of the atom in roman numerals (e.g., 'I', 'II')
	wave_start: float
		the minimum observed wavelength 
	wave_end: float
		the maximum observed wavelength
	redshift: float
		redshift of the absorption line
	
	Return:
	----------
	transitions_params_array: array_like
		[oscilator strength, rest wavelength, dammping coefficient, mass of the atom] for all transitions within the wavelength interval. shape = (number of transitions, 4) 
	"""
	amu = 1.66053892e-24   # 1 atomic mass in grams

	# Absolute path for BayseVP
	data_path = os.path.dirname(os.path.abspath(__file__)) 
	data_file = data_path + '/data/atom.dat'
	atoms,states  = np.loadtxt(data_file, dtype=str,unpack=True,usecols=[0,1])
	wave,osc_f,Gamma,mass = np.loadtxt(data_file,unpack=True,usecols=[2,3,4,5])
	mass = mass*amu 

	inds = np.where((atoms == atom) & 
					(states == state) & 
					(wave >= wave_start/(1+redshift)) & 
					(wave < wave_end/(1+redshift)))[0]
 
	if len(inds) == 0:
		return np.empty(4)*np.nan
	else:
		return np.array([osc_f[inds],wave[inds],Gamma[inds], mass[inds]]).T


def printline():
	print("---------------------------------------------------------------------")


def get_bayesvp_Dir():
	return os.path.dirname(os.path.realpath(__file__))


class MyParser(argparse.ArgumentParser):
	def error(self, message):
		sys.stderr.write('error: %s\n' % message)
		self.print_help()
		sys.exit(2)

def print_config_help():
    
	printline()
	printline()
	print('Example of config file content:')
	printline()
	print('spec_path test_path_to_spec')
	print('output o6')
	print('mcmc 100 200 4 bic kombine')
	print('%% OVI.spec 1030.000000 1033.000000')
	print('% O VI 15 30 0.000000')
	print('logN 10.00 18.00')
	print('b    0.00 100.00')
	print('z    0.000000 300.00')
	printline()
	printline()
	print('\n')

	printline()
	printline()
	print('Structure of a line in config file: ')
	print('parameter_type parameter(s)')
	printline()
	print('spec_path full_path_to_spectrum')
	print('output mcmc_chain_filename')
	print('mcmc walkers steps_per_walker parallel_threads MCMC_evidence_criterion MCMC_sampler')
	print('%% spectrum_file_name wave_begin1 wave_end1 wave_begin2 wave_end2 ...')
	print('% Atom State logN_guess b_guess z_guess')
	print('logN min_logN max_logN')
	print('b    min_b max_b')
	print('z    center_z dv_range')
	printline()
	printline()

	print('\n')

	printline()
	printline()
	print('optional lines/commands:')

	printline()
	print('lsf lsf_file_name\n')
	print('where lsf = line spread function to be convolved with model')
	print('lsf_file_name is assumed to be in spec_path/database')

	printline()
	print('continuum n \n')
	print('where n = polynomial degree; number of parameter = n+1')




	printline()
	print('! auto n\n')
	print('Automatically try up to n number of Voigt profile component')

	printline()
	print('# lines begin with # is an optional comment')
	printline()



import unittest
import os
import sys
import numpy as np

from bayesvp.config import DefineParams
from bayesvp.likelihood import Posterior
from bayesvp.utilities import get_bayesvp_Dir

###############################################################################
# TEST CASE 1: OVI line with stock config file and spectrum
###############################################################################

class TCPosterior(unittest.TestCase):

    def setUp(self):

        # read example config file
        code_path = get_bayesvp_Dir()
        self.config_ex = code_path + '/data/example/config_OVI.dat'
        self.config_params = DefineParams(self.config_ex)
        self.posterior = Posterior(self.config_params)

    ###########################################################################
    # Basic Tests for likelihood, prior and posterior
    ###########################################################################

    def test_default_no_continuum(self):
        self.assertFalse(self.config_params.cont_normalize)

    def test_lnlike(self):
        vp_params = np.array([15,20,0]) # logN, b, z
        correct = -344.55470583729573
        self.assertEqual(self.posterior.lnlike(vp_params),correct)

    def test_prior(self):
        vp_params = np.array([15,20,0])
        correct = 0 
        self.assertEqual(self.posterior.lnprior(vp_params),correct)

        # Outside of prior (logN)
        vp_params = np.array([19,20,0])
        correct = -np.inf
        self.assertEqual(self.posterior.lnprior(vp_params),correct)

        # Outside of prior (b)
        vp_params = np.array([15,-10,0])
        correct = -np.inf
        self.assertEqual(self.posterior.lnprior(vp_params),correct)

        # Outside of prior (z)
        vp_params = np.array([10,20,-1])
        correct = -np.inf
        self.assertEqual(self.posterior.lnprior(vp_params),correct)


    def test_call(self):
        vp_params = np.array([15,20,0])
        correct = -344.55470583729573
        self.assertEqual(self.posterior.__call__(vp_params),correct)

        vp_params = np.array([10,20,-1])
        correct = -np.inf
        self.assertEqual(self.posterior.__call__(vp_params),correct)



if __name__ == '__main__':
    unittest.main()


import unittest
import os
import sys
import numpy as np

from bayesvp.config import DefineParams
from bayesvp.utilities import get_bayesvp_Dir

###############################################################################
# TEST CASE 1: OVI line with stock config file and spectrum


# config file, bayesvp/data/example/bvp_configs/config_OVI.dat,
# should contain the following:

# spec_path test_path_to_spec
# output o6
# mcmc 100 200 4 bic kombine
# %% OVI.spec 1030.000000 1033.000000
# % O VI 15 30 0.000000
# logN 10.00 18.00
# b    0.00 100.00
# z    0.000000 300.00
###############################################################################

class TCConfigFile(unittest.TestCase):

    def setUp(self):
        code_path = get_bayesvp_Dir()
        # read example config file
        self.config_ex = code_path + '/data/example/config_OVI.dat'
        self.config_params = DefineParams(self.config_ex)

    def test_config_file_exists(self):
        self.assertTrue(os.path.isfile(self.config_ex))

    def test_default_no_continuum_params(self):
        self.assertFalse(self.config_params.cont_normalize)
        self.assertEqual(self.config_params.cont_nparams,0)


    def test_spectral_data(self):

        self.assertTrue(os.path.isfile(self.config_params.spec_fname))

        self.assertEqual(len(self.config_params.flux),len(self.config_params.dflux))
        self.assertEqual(len(self.config_params.wave),len(self.config_params.dflux))

        self.assertFalse(np.isnan(self.config_params.wave).any())
        self.assertFalse(np.isnan(self.config_params.flux).any())
        self.assertFalse(np.isnan(self.config_params.dflux).any())



    def test_example_mcmc_params(self):
        self.assertEqual(self.config_params.mcmc_sampler,'kombine')
        self.assertEqual(self.config_params.model_selection,'bic')
        
        self.assertEqual(self.config_params.n_component,1)
        self.assertEqual(self.config_params.n_params,self.config_params.n_component*3)
        
        self.assertEqual(self.config_params.nsteps,200)
        self.assertEqual(self.config_params.nthreads,4)
        self.assertEqual(self.config_params.nwalkers,100)

    def test_example_priors(self):
        self.assertEqual(np.shape(self.config_params.priors),(3,2))

        self.assertAlmostEqual(self.config_params.priors[0][0],10.0)
        self.assertAlmostEqual(self.config_params.priors[0][1],18.0)

        self.assertAlmostEqual(self.config_params.priors[1][0],0.0)
        self.assertAlmostEqual(self.config_params.priors[1][1],100.0)

        self.assertAlmostEqual(self.config_params.priors[2][0],-0.00100069)
        self.assertAlmostEqual(self.config_params.priors[2][1],0.00100069)

        self.assertEqual(float(self.config_params.redshift),0.0)

    def test_transistion_params_array(self):
        self.assertFalse(np.isnan(self.config_params.transitions_params_array).any())
        self.assertEqual(np.shape(self.config_params.transitions_params_array),(1,1,1,4))
        self.assertIsNotNone(self.config_params.transitions_params_array)
        self.assertFalse(np.isnan(self.config_params.transitions_params_array[0][0][0][1]))

    def test_vp_params(self):
        self.assertEqual(self.config_params.wave_begins,1030.0)
        self.assertEqual(self.config_params.wave_ends,1033.0)

        self.assertEqual(self.config_params.vp_params_flags[0],0)
        self.assertEqual(self.config_params.vp_params_flags[1],1)
        self.assertEqual(self.config_params.vp_params_flags[2],2)

        self.assertEqual(self.config_params.vp_params_type[0],'logN')
        self.assertEqual(self.config_params.vp_params_type[1],'b')
        self.assertEqual(self.config_params.vp_params_type[2],'z')

        self.assertEqual(self.config_params.vp_params[0][0],'15')
        self.assertEqual(self.config_params.vp_params[0][1],'30')
        self.assertEqual(self.config_params.vp_params[0][2],'0.000000')


if __name__ == '__main__':

    unittest.main()


################################################################################
#
# vp_model.py   	    (c) Cameron Liang 
#						University of Chicago
#     				    jwliang@oddjob.uchicago.edu
#
# Voigt profile model; produce prediction of model given the config file   
# (specifically the initial parameters). It can be imported to produce 
# a voigt profile on demand specified by user as well. 
################################################################################

import numpy as np
from scipy.special import wofz
import sys
import os

from bayesvp.utilities import convolve_lsf,get_transitions_params

# constants [cgs units]
h  = 6.6260755e-27   # planck constant
kB = 1.380658e-16    # Boltzmann constant
c  = 2.99792458e10   # speed of light
m  = 9.10938291e-28  # electron mass
mH = 1.67e-24        # proton mass
e  = 4.80320425e-10  # electron charge
sigma0 = 0.0263      # Cross section [cm^2/sec]

# Units Conversion
cm_km = 1.e-5  # Convert cm to km
km_cm = 1.e5   # Convert km to cm
ang_cm = 1.e-8 # Convert Angstrom to cm
kpc_cm = 3.0856e21 # Convert kpc to cm 

def wavelength_array(wave_start, wave_end, dv):
    """
    Create Wavelength array with resolution dv
    
    Parameters:
    ----------
    wave_start: float
        starting wavelength of array [A]
    wave_end: float
        ending wavelength of array [A]
    dv: float
        resolution element [km/s]
    
    Returns
    ----------
    wavelength_array: array_like
    """ 
    
    c = 299792.458       # speed of light  [km/s]
    wave_start = float(wave_start); wave_end = float(wave_end)

    # Calcualte Total number of pixel given the resultion and bounds of spectrum
    total_number_pixel = np.int(np.log10(wave_end/wave_start) / 
                                np.log10(1 + dv/c) + 0.5)
    array_index        = np.arange(0,total_number_pixel,1)
    
    # Return wavelength array
    return wave_start * ((1 + dv/c)**array_index)

def voigt_shape(x, a):
    """
    Real part of Faddeeva function, where    
    w(z) = exp(-z^2) erfc(jz)
    """
    z = x + 1j*a
    return wofz(z).real

def voigt_profile_line(b, z, nu, nu0, Gamma):
    """
    Generate Voigt Profile for a given transition

    Parameters:
    ----------
    b: float
        b parameter of the voigt profile
    z: float
        resfhit of the absorption line
    nu: array_like
        rest frame frequncy array
    nu0: float
        rest frame frequency of transition [1/s]
    Gamma: float
        Damping coefficient (transition specific)

    Returns:
    ----------
    V: array_like
        voigt profile as a function of frequency
    """

    delta_nu = nu - nu0 / (1+z)
    delta_nuD = b * nu / c
    
    prefactor = 1.0 / ((np.pi**0.5)*delta_nuD)
    x = delta_nu/delta_nuD
    a = Gamma/(4*np.pi*delta_nuD)

    return prefactor * voigt_shape(x,a)  

def b_parameter(logT, b_nt, mass):
    """
    Combined thermal and non-thermal velocity

    Parameters:
    ----------
    mass: float
        mass of ion in the transition [grams]
    logT: array_like
        log10 of Temperature [Kelvin]
    b_nt: array_like
        non-thermal velocity dispersion [km/s]

    Returns:
    ----------
    b parameter: array_like; [km/s]
    """
    
    temp = 10**logT
    b_thermal = np.sqrt(2. * kB*temp / mass)*cm_km  # [km/s]
    return np.sqrt(b_thermal**2 + b_nt**2)


def general_intensity(logN, b, z, wave, atomic_params):
    """
    Takes a general combination of atomic 
    parameters, without specifying the name of the transition 
    to compute the flux
    
    Parameters:
    ----------
    logN: float
        log10 of column density [cm-2] 
    b: float
        total b parameter [km/s]
    z: float
        redshift of system
    wave: 1D array
        observed wavelength array
    atomic_params: array
        array of oscillator strength, rest frame wavelength [A], damping coefficient, mass [grams] of the transition

    Returns:
    ----------
    intensity: array
        normalized flux of the spectrum    
    """
    f,wave0,gamma,mass = atomic_params

    # Convert to cgs units
    b       = b * km_cm       # Convert km/s to cm/s
    N       = 10**logN        # Column densiy in linear space
    lambda0 = wave0*ang_cm    # Convert Angstrom to cm
    nu0     = c/lambda0       # Rest frame frequency 
    nu      = c/(wave*ang_cm) # Frequency array 

    # Compute Optical depth
    tau = N*sigma0*f*voigt_profile_line(b,z,nu,nu0,gamma)

    # Return Normalized intensity
    return np.exp(-tau.astype(np.float))

def simple_spec(logN, b, z, wave, atom=None, state=None, lsf=1):
    """
    Generate a single component absorption for all transitions
    within the given observed wavelength, redshift for the desired
    atom and state

    Parameters:
    ----------
    logN: float
        log10 of column density [cm-2] 
    b: float
        total b parameter [km/s]
    z: float
        redshift of system
    wave: 1D array
        observed wavelength array

    Returns:
    ----------
    intensity: array
        normalized flux of the spectrum    
    """

    if atom is None or state is None:
        print('Enter atom and ionization state (e.g., C IV)\n')
        exit()

    atomic_params = get_transitions_params(atom,state,min(wave),max(wave),z)
    n_transitions = len(atomic_params)
    
    spec = []
    for l in xrange(n_transitions):
        if not np.isnan(atomic_params[l]).any():
            model_flux = general_intensity(logN,b,z,wave,atomic_params[l]) 
            spec.append(convolve_lsf(model_flux,lsf)) 
        else:
            return np.ones(wave)
    # Return the convolved model flux with LSF
    return np.product(spec,axis=0)

def generic_prediction(alpha, obs_spec_obj):
    """
    This is a model flux produced by a generic combination 
    of the voigt profile parameters by arbitrary combination
    of fixing/tieing/freeing any parameters; see config.py

    Parameters:
    ----------
    alpha: array_like
        One dimensional array of n parameters; The structure 
        of the array needs to match the ones specified in the 
        config file  
    obs_spec_obj:
        Paramters object defined by the config file
        see ./config.py
    
    Returns:
    ----------
    Model flux: 1D array;
        Predicted flux based on the paramters with length equal to 
        the length of the input wavelength array
    """
    component_flags = obs_spec_obj.vp_params_flags.reshape(obs_spec_obj.n_component,3)

    spec = [] 
    for i in xrange(obs_spec_obj.n_component):
        # Re-group parameters intro [logN, b,z] for each component

        temp_alpha = np.zeros(3)
        for j in xrange(3):
            # NaN indicates parameter has been fixed
            if np.isnan(component_flags[i][j]): 
                # access the fixed value from vp_params after removing the upper case letter 
                temp_alpha[j] = float(obs_spec_obj.vp_params[i][j][:-1])
            else: 
                # access the index map from flags to alpha 
                temp_alpha[j] = alpha[int(component_flags[i][j])] 


        # Compute spectrum for each component, region, and transition.
        n_wavelength_regions = len(obs_spec_obj.wave_begins)
 
        for k in xrange(n_wavelength_regions):  
            n_transitions = len(obs_spec_obj.transitions_params_array[i][k])
            for l in xrange(n_transitions):
                if not np.isnan(obs_spec_obj.transitions_params_array[i][k][l]).any():
                    model_flux = general_intensity(temp_alpha[0],temp_alpha[1],temp_alpha[2],obs_spec_obj.wave,obs_spec_obj.transitions_params_array[i][k][l])

                    # Convolve (potentially )LSF for each region 
                    spec.append(convolve_lsf(model_flux,obs_spec_obj.lsf[k])) 
    
    # Return the convolved model flux with LSF
    return np.product(spec,axis=0)

def poly_continuum(wave,flux, *params):
    # arbitrary polynomial continuum
    x = wave-np.median(wave)
    return sum([p*(x**i) for i, p in enumerate(params)]) + np.median(flux)

def continuum_model_flux(alpha,obs_spec_obj):
    """
    Model function that includes continuum (linear order)
    """
    
    if obs_spec_obj.cont_normalize:
        num_boundary = obs_spec_obj.cont_nparams
        model_flux = generic_prediction(alpha[:-num_boundary],obs_spec_obj)
        local_continuum = poly_continuum(obs_spec_obj.wave,obs_spec_obj.flux,*alpha[-num_boundary:])
        return model_flux * local_continuum
    else:
        model_flux = generic_prediction(alpha,obs_spec_obj)
        return model_flux



import unittest
import os
import sys
import numpy as np


from bayesvp.vp_model import general_intensity
from bayesvp.vp_model import wavelength_array
from bayesvp.vp_model import simple_spec

###############################################################################
# TEST CASE 1: OVI line with stock config file and spectrum
###############################################################################

class TCSingleVP(unittest.TestCase):

    def setUp(self):
        # OVI line
        self.logN = 14.5; self.b = 20; self.z = 0
        self.fosc = 1.32500000e-01; self.wave0 = 1.03192610e+03
        self.gamma = 4.14900000e+08; self.mass_oxy = 2.65676264e-23
        self.atomic_params = np.array([self.fosc,self.wave0,self.gamma,self.mass_oxy])

    def test_general_intensity(self):

        norm_intensity = general_intensity(self.logN, self.b, self.z, 
                        self.wave0, self.atomic_params)
        self.assertAlmostEqual(norm_intensity,0.040468,places=5)


    def test_simple_spec(self):
        wave = wavelength_array(1031, 1033, 7.5)
        flux = simple_spec(self.logN,self.b,self.z,wave,'O','VI')
        self.assertEqual(np.shape(wave),(77,))
        self.assertEqual(np.shape(flux),np.shape(wave))


if __name__ == '__main__':

    unittest.main()


################################################################################
#
# write_bvp_config.py 	(c) Cameron Liang 
#						University of Chicago
#     				    jwliang@oddjob.uchicago.edu
#
# Interactive script to aid writing a config file for voigt profile fitting. 
#
################################################################################

import os
import sys
import numpy as np
from bayesvp.utilities import MyParser

class WriteBayesVPConfig:

    def __init__(self):
        self.spec_path    = 'test_path_to_spec'
        self.spec_fname   = 'OVI.spec'
        self.output_chain_fname = 'o6'
        self.atom         = 'O'
        self.state        = 'VI'
        self.auto         = 1
        self.wave_start   = 1030.0
        self.wave_end     = 1033.0

        self.nwalkers     = 100
        self.nsteps       = 200
        self.nthreads     = 4

        self.min_logN = 10.0;       self.max_logN       = 18
        self.min_b    = 0;          self.max_b          = 100
        self.central_redshift = 0.; self.velocity_range = 300

        self.model_selection = 'bic'
        self.mcmc_sampler    = 'kombine'

    def interactive_var(self):
        self.spec_path    = raw_input('Path to spectrum:\n')
        self.spec_fname   = raw_input('Spectrum filename: ')
        self.output_chain_fname = raw_input('filename for output chain: ')
        self.atom         = raw_input('atom: ')
        self.state        = raw_input('state: ')
        self.auto         = int(raw_input('Maximum number of components to try: '))
        self.wave_start   = float(raw_input('Starting wavelength: '))
        self.wave_end     = float(raw_input('Ending wavelength: '))
        
        print('\nNow enter the priors. Press Enter for default values.')
        self.min_logN = 0; self.max_logN = 24 

        try:
            self.min_logN = float(raw_input('min logN = '))
        except ValueError:
            self.min_logN = 0

        try:
            self.max_logN = float(raw_input('max logN = '))
        except ValueError:
            self.max_logN = 24

        try:
            self.min_b = float(raw_input('min b = '))
        except ValueError:
            self.min_b = 0

        try:
            self.max_b = float(raw_input('max b = '))
        except ValueError:
            self.max_b = 100

        try:
            self.central_redshift = float(raw_input('central redshift = '))
        except ValueError:
            self.central_redshift = 0

        try:
            self.velocity_range   = float(raw_input('velocity range [km/s] = '))
        except ValueError:
            self.velocity_range = 300

        print('\nNow enter the MCMC parameters..')
        try:
            self.nwalkers = int(raw_input('Number of walkers: '))
        except ValueError:
            self.nwalkers = 100
        
        try:
            self.nsteps   = int(raw_input('Number of steps:  '))
        except ValueError:
            self.nsteps = 200

        try:
            self.nthreads = int(raw_input('Number of processes: '))
        except ValueError:
            self.nthreads = 4

        self.model_selection = raw_input('Model selection method bic(default),aic,bf: ')
        if self.model_selection == '':
            self.model_selection = 'bic'
        
        self.mcmc_sampler = raw_input('MCMC sampler kombine(default), emcee: ')
        if self.mcmc_sampler == '':
            self.mcmc_sampler = 'kombine'


    def print_to_file(self,interactive_write=False):

        if interactive_write:
            self.interactive_var()

        if self.spec_path == 'test_path_to_spec':
            self.config_path = './bvp_configs'

        else:
            self.config_path = self.spec_path + '/bvp_configs'

        if not os.path.isdir(self.config_path):
            os.mkdir(self.config_path)

        self.config_fname = self.config_path + '/config_' + self.atom + self.state + '.dat'
        f = open(self.config_fname,'w')
        f.write('spec_path %s\n' % self.spec_path)
        if self.auto > 1:
            f.write('! auto %d\n' % self.auto)
        f.write('output %s\n' % self.output_chain_fname)
        f.write('mcmc %d %d %d %s %s\n' % (self.nwalkers,self.nsteps,self.nthreads,
                                    self.model_selection,self.mcmc_sampler))
        f.write('%%%% %s %.6f %.6f\n' % (self.spec_fname,self.wave_start,self.wave_end))
        f.write('%% %s %s 15 30 %f\n' % (self.atom, self.state,self.central_redshift))

        f.write('logN %.2f %.2f\n' % (self.min_logN,    self.max_logN))
        f.write('b    %.2f %.2f\n' % (self.min_b,       self.max_b))
        f.write('z    %.6f %.2f\n' % (self.central_redshift,self.velocity_range))
        f.close()

        print('Written config file: %s\n' % self.config_fname)


def main():

    config_writer = WriteBayesVPConfig()

    parser = MyParser()
    parser.add_argument("-a", "--auto",help="write default config file",
                        action="store_true")
    parser.add_argument("-i", "--interactive",help="write config file based on user input",
                        action="store_true")
    
    args = parser.parse_args()

    if len(sys.argv)==1:
        parser.print_help(sys.stderr)
        sys.exit(1)
    
    if args.auto:
        config_writer.print_to_file()

    if args.interactive:
        config_writer.print_to_file(args.interactive)


if __name__ == '__main__':
    sys.exit(main() or 0)


import sys
import os

from bayesvp.mcmc_setup import bvp_mcmc
from bayesvp.utilities import MyParser

def main(config_fname = None):
    parser = MyParser()
    parser.add_argument('config_fname',help="full path to config filename", nargs='?')
    parser.add_argument("-t", "--test",help="a mcmc fitting test with default config and spectrum",
                        action="store_true")
    parser.add_argument("-v", "--verbose",help="print stdout to file",
                        action="store_true")
    parser.add_argument("-pc", "--printconfig",help="print config parameters to screen",
                        action="store_true")
    
    if len(sys.argv)==1:
        parser.print_help(sys.stderr)
        sys.exit(1)
    
    args = parser.parse_args()

    if args.test:
        from bayesvp.utilities import get_bayesvp_Dir
        path = get_bayesvp_Dir()
        config_fname = path + '/data/example/config_OVI.dat'
        args.printconfig = True
        bvp_mcmc(config_fname,args.printconfig)

    if args.config_fname: 
        if os.path.isfile(args.config_fname):
            bvp_mcmc(args.config_fname,args.printconfig)
        else:
            sys.exit('Config file does not exist:\n %s' % args.config_fname)


    """
    if args.config_fname is None: 
        pass
    else:
        if os.path.isfile(args.config_fname):
            bvp_mcmc(args.config_fname,args.printconfig)
        else:
            print('Config file does not exist:\n %s' % args.config_fname)
    """
if __name__ == '__main__':
    sys.exit(main() or 0)



import io
from setuptools import setup

with io.open('README.rst', encoding = 'utf-8') as f:
	long_description = f.read()

setup(name='bayesvp',
    version='0.1.2',
    description='Bayesian MCMC Voigt Profile Fitting',
    long_description = long_description,
    url='https://github.com/cameronliang/bayesvp',
    author='Cameron Liang',
    author_email='cameron.liang@gmail.com',
    license='MIT',
    packages=['bayesvp'],
    include_package_data=True,
    entry_points={
        'console_scripts': ['bvpfit = bayesvp.scripts.bvpfit:main',
        'bvp_write_config = bayesvp.scripts.bvp_write_config:main',
        'bvp_process_model = bayesvp.scripts.bvp_process_model:main'],
    },
    install_requires=[
        'numpy', 'scipy',
        'matplotlib', 'corner',
        'kombine','emcee', 'sklearn'
        ],
    zip_safe=False)



import unittest

from bayesvp.tests import test_config
from bayesvp.tests import test_likelihood
from bayesvp.tests import test_model

suites = []

suites.append(unittest.TestLoader().loadTestsFromTestCase(test_config.TCConfigFile))
suites.append(unittest.TestLoader().loadTestsFromTestCase(test_likelihood.TCPosterior))
suites.append(unittest.TestLoader().loadTestsFromTestCase(test_model.TCSingleVP))


# Run tests
suite = unittest.TestSuite(suites)
unittest.TextTestRunner(verbosity = 2).run(suite)
